/*_############################################################################
  _## 
  _##  SNMP4J - SocketTimeout.java  
  _## 
  _##  Copyright (C) 2003-2018  Frank Fock and Jochen Katz (SNMP4J.org)
  _##  
  _##  Licensed under the Apache License, Version 2.0 (the "License");
  _##  you may not use this file except in compliance with the License.
  _##  You may obtain a copy of the License at
  _##  
  _##      http://www.apache.org/licenses/LICENSE-2.0
  _##  
  _##  Unless required by applicable law or agreed to in writing, software
  _##  distributed under the License is distributed on an "AS IS" BASIS,
  _##  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  _##  See the License for the specific language governing permissions and
  _##  limitations under the License.
  _##  
  _##########################################################################*/

package org.snmp4j.transport;

import org.snmp4j.log.LogAdapter;
import org.snmp4j.log.LogFactory;
import org.snmp4j.mp.SnmpConstants;

import java.io.IOException;
import java.util.TimerTask;

class SocketTimeout extends TimerTask {

  private static final LogAdapter logger = LogFactory.getLogger(SocketTimeout.class);

  private TcpTransportMapping tcpTransportMapping;
  private ServerSocketEntry entry;

  public SocketTimeout(TcpTransportMapping tcpTransportMapping, ServerSocketEntry entry) {
    this.tcpTransportMapping = tcpTransportMapping;
    this.entry = entry;
  }

  /**
   * run
   */
  public void run() {
    long now = System.nanoTime();
    if ((tcpTransportMapping.socketCleaner == null) ||
        ((now - entry.getLastUse())/SnmpConstants.MILLISECOND_TO_NANOSECOND >= tcpTransportMapping.connectionTimeout)) {
      if (logger.isDebugEnabled()) {
        logger.debug("Socket has not been used for "+
                     (now - entry.getLastUse())+
                     " milliseconds, closing it");
      }
      tcpTransportMapping.sockets.remove(entry.getPeerAddress());
      ServerSocketEntry entryCopy = entry;
      try {
        synchronized (entryCopy) {
          entryCopy.getSocket().close();
        }
        logger.info("Socket to "+entryCopy.getPeerAddress()+
                    " closed due to timeout");
      }
      catch (IOException ex) {
        logger.error(ex);
      }
    }
    else {
      long nextRun = System.currentTimeMillis() +
          (now - entry.getLastUse())/SnmpConstants.MILLISECOND_TO_NANOSECOND + tcpTransportMapping.connectionTimeout;
      if (logger.isDebugEnabled()) {
        logger.debug("Scheduling " + nextRun);
      }
      SocketTimeout socketTimeout = new SocketTimeout(tcpTransportMapping, entry);
      entry.setSocketTimeout(socketTimeout);
      tcpTransportMapping.socketCleaner.schedule(socketTimeout, nextRun);
    }
  }

  public boolean cancel(){
      boolean result = super.cancel();
      // free objects early
      entry = null;
      return result;
  }
}
